@@ -10,14 +10,19 @@ import sys
 import tempfile
 import time
 import unittest
+from tools import emprofile
+from tools import utils
 
 import common
+from common import errlog
 
-from tools.shared import cap_max_workers_in_pool
+from tools.colored_logger import with_color, CYAN, GREEN, RED
+from tools.utils import WINDOWS
 
 
 NUM_CORES = None
 seen_class = set()
+torn_down = False
 
 
 # Older Python versions have a bug with multiprocessing shared data
@@ -28,12 +33,28 @@ def python_multiprocessing_structures_are_buggy():
   return (v.major, v.minor, v.micro) <= (3, 12, 7) or (v.major, v.minor, v.micro) == (3, 13, 0)
 
 
-def run_test(test, failfast_event, lock, progress_counter, num_tests):
-  # If failfast mode is in effect and any of the tests have failed,
-  # and then we should abort executing further tests immediately.
-  if failfast_event and failfast_event.is_set():
+def cap_max_workers_in_pool(max_workers, is_browser):
+  if is_browser and 'EMTEST_CORES' not in os.environ and 'EMCC_CORES' not in os.environ:
+    # TODO experiment with this number. In browser tests we'll be creating
+    # a browser instance per worker which is expensive.
+    max_workers = max_workers // 2
+  # Python has an issue that it can only use max 61 cores on Windows: https://github.com/python/cpython/issues/89240
+  if WINDOWS:
+    return min(max_workers, 61)
+  return max_workers
+
+
+def run_test(test, allowed_failures_counter, lock, progress_counter, num_tests):
+  # If we have exceeded the number of allowed failures during the test run,
+  # abort executing further tests immediately.
+  if allowed_failures_counter and allowed_failures_counter.value < 0:
     return None
 
+  def test_failed():
+    if allowed_failures_counter is not None:
+      with lock:
+        allowed_failures_counter.value -= 1
+
   olddir = os.getcwd()
   result = BufferedParallelTestResult(lock, progress_counter, num_tests)
   temp_dir = tempfile.mkdtemp(prefix='emtest_')
@@ -45,14 +66,13 @@ def run_test(test, failfast_event, lock, progress_counter, num_tests):
     test(result)
 
     # Alert all other multiprocess pool runners that they need to stop executing further tests.
-    if failfast_event is not None and result.test_result not in ['success', 'skipped']:
-      failfast_event.set()
+    if result.test_result not in ['success', 'skipped']:
+      test_failed()
   except unittest.SkipTest as e:
     result.addSkip(test, e)
   except Exception as e:
     result.addError(test, e)
-    if failfast_event is not None:
-      failfast_event.set()
+    test_failed()
   # Before attempting to delete the tmp dir make sure the current
   # working directory is not within it.
   os.chdir(olddir)
@@ -60,6 +80,18 @@ def run_test(test, failfast_event, lock, progress_counter, num_tests):
   return result
 
 
+# Executes the teardown process once. Returns True if the teardown was
+# performed, False if it was already torn down.
+def tear_down():
+  global torn_down
+  if torn_down:
+    return False
+  torn_down = True
+  for cls in seen_class:
+    cls.tearDownClass()
+  return True
+
+
 class ParallelTestSuite(unittest.BaseTestSuite):
   """Runs a suite of tests in parallel.
 
@@ -69,7 +101,7 @@ class ParallelTestSuite(unittest.BaseTestSuite):
   def __init__(self, max_cores, options):
     super().__init__()
     self.max_cores = max_cores
-    self.failfast = options.failfast
+    self.max_failures = options.max_failures
     self.failing_and_slow_first = options.failing_and_slow_first
 
   def addTest(self, test):
@@ -84,23 +116,41 @@ class ParallelTestSuite(unittest.BaseTestSuite):
     # issues.
     # multiprocessing.set_start_method('spawn')
 
-    # If we are running with --failing-and-slow-first, then the test list has been
-    # pre-sorted based on previous test run results. Otherwise run the tests in
-    # reverse alphabetical order.
-    tests = list(self if self.failing_and_slow_first else self.reversed_tests())
-    use_cores = cap_max_workers_in_pool(min(self.max_cores, len(tests), num_cores()))
-    print('Using %s parallel test processes' % use_cores, file=sys.stderr)
+    tests = self.get_sorted_tests()
+    contains_browser_test = any(test.is_browser_test() for test in tests)
+    use_cores = cap_max_workers_in_pool(min(self.max_cores, len(tests), num_cores()), contains_browser_test)
+    errlog(f'Using {use_cores} parallel test processes')
     with multiprocessing.Manager() as manager:
-      pool = multiprocessing.Pool(use_cores)
-      if python_multiprocessing_structures_are_buggy():
-        failfast_event = progress_counter = lock = None
-      else:
-        failfast_event = manager.Event() if self.failfast else None
-        progress_counter = manager.Value('i', 0)
-        lock = manager.Lock()
-      results = [pool.apply_async(run_test, (t, failfast_event, lock, progress_counter, len(tests))) for t in tests]
-      results = [r.get() for r in results]
-      results = [r for r in results if r is not None]
+      # Give each worker a unique ID.
+      worker_id_counter = manager.Value('i', 0) # 'i' for integer, starting at 0
+      worker_id_lock = manager.Lock()
+      with multiprocessing.Pool(
+        processes=use_cores,
+        initializer=common.init_worker,
+        initargs=(worker_id_counter, worker_id_lock),
+      ) as pool:
+        if python_multiprocessing_structures_are_buggy():
+          # When multiprocessing shared structures are buggy we don't support failfast
+          # or the progress bar.
+          allowed_failures_counter = progress_counter = lock = None
+          if self.max_failures < 2**31 - 1:
+            errlog('The version of python being used is not compatible with --failfast and --max-failures options. See https://github.com/python/cpython/issues/71936')
+            sys.exit(1)
+        else:
+          allowed_failures_counter = manager.Value('i', self.max_failures)
+          progress_counter = manager.Value('i', 0)
+          lock = manager.Lock()
+        results = pool.starmap(run_test, ((t, allowed_failures_counter, lock, progress_counter, len(tests)) for t in tests), chunksize=1)
+        # Send a task to each worker to tear down the browser and server. This
+        # relies on the implementation detail in the worker pool that all workers
+        # are cycled through once.
+        num_tear_downs = sum([pool.apply(tear_down, ()) for i in range(use_cores)])
+        # Assert the assumed behavior above hasn't changed.
+        if num_tear_downs != use_cores:
+          errlog(f'Expected {use_cores} teardowns, got {num_tear_downs}')
+
+    # Filter out the None results which can occur if # of allowed errors was exceeded and the harness aborted.
+    results = [r for r in results if r is not None]
 
     if self.failing_and_slow_first:
       previous_test_run_results = common.load_previous_test_run_results()
@@ -124,32 +174,63 @@ class ParallelTestSuite(unittest.BaseTestSuite):
         update_test_results_to(r.test_name.split(' ')[0])
 
       json.dump(previous_test_run_results, open(common.PREVIOUS_TEST_RUN_RESULTS_FILE, 'w'), indent=2)
-    pool.close()
-    pool.join()
+
     return self.combine_results(result, results)
 
-  def reversed_tests(self):
-    """A list of this suite's tests, sorted reverse alphabetical order.
+  def get_sorted_tests(self):
+    """A list of this suite's tests, sorted with the @is_slow_test tests first.
 
-    Many of the tests in test_core are intentionally named so that long tests
-    fall toward the end of the alphabet (e.g. test_the_bullet). Tests are
-    loaded in alphabetical order, so here we reverse that in order to start
-    running longer tasks earlier, which should lead to better core utilization.
-
-    Future work: measure slowness of tests and sort accordingly.
+    Future work: measure and store the speed of tests each test sort more accurately.
     """
-    return sorted(self, key=str, reverse=True)
+    if self.failing_and_slow_first:
+      # If we are running with --failing-and-slow-first, then the test list has been
+      # pre-sorted based on previous test run results (see `runner.py`)
+      return list(self)
+
+    def test_key(test):
+      testMethod = getattr(test, test._testMethodName)
+      is_slow = getattr(testMethod, 'is_slow', False)
+      return (is_slow, str(test))
+
+    return sorted(self, key=test_key, reverse=True)
 
   def combine_results(self, result, buffered_results):
-    print('', file=sys.stderr)
-    print('DONE: combining results on main thread', file=sys.stderr)
-    print('', file=sys.stderr)
+    errlog('')
+    errlog('DONE: combining results on main thread')
+    errlog('')
     # Sort the results back into alphabetical order. Running the tests in
     # parallel causes mis-orderings, this makes the results more readable.
     results = sorted(buffered_results, key=lambda res: str(res.test))
     result.core_time = 0
+
+    # shared data structures are hard in the python multi-processing world, so
+    # use a file to share the flaky test information across test processes.
+    flaky_tests = open(common.flaky_tests_log_filename).read().split() if os.path.isfile(common.flaky_tests_log_filename) else []
+    # Extract only the test short names
+    flaky_tests = [x.split('.')[-1] for x in flaky_tests]
+
+    # The next updateResult loop will print a *lot* of lines really fast. This
+    # will cause a Python exception being thrown when attempting to print to
+    # stderr, if stderr is in nonblocking mode, like it is on Buildbot CI:
+    # See https://github.com/buildbot/buildbot/issues/8659
+    # To work around that problem, set stderr to blocking mode before printing.
+    if not WINDOWS:
+      os.set_blocking(sys.stderr.fileno(), True)
+
     for r in results:
+      # Merge information of flaky tests into the test result
+      if r.test_result == 'success' and r.test_short_name() in flaky_tests:
+        r.test_result = 'warnings'
+      # And integrate the test result to the global test object
       r.updateResult(result)
+
+    # Generate the parallel test run visualization
+    if os.getenv('EMTEST_VISUALIZE'):
+      emprofile.create_profiling_graph(utils.path_from_root('out/graph'))
+      # Cleanup temp files that were used for the visualization
+      emprofile.delete_profiler_logs()
+      utils.delete_file(common.flaky_tests_log_filename)
+
     return result
 
 
@@ -166,11 +247,18 @@ class BufferedParallelTestResult:
     self.lock = lock
     self.progress_counter = progress_counter
     self.num_tests = num_tests
+    self.failures = []
+    self.errors = []
 
   @property
   def test(self):
     return self.buffered_result.test
 
+  def test_short_name(self):
+    # Given a test name e.g. "test_atomic_cxx (test_core.core0.test_atomic_cxx)"
+    # returns a short form "test_atomic_cxx" of the test.
+    return self.test_name.split(' ', 1)[0]
+
   def addDuration(self, test, elapsed):
     self.test_duration = elapsed
 
@@ -182,6 +270,30 @@ class BufferedParallelTestResult:
     self.buffered_result.updateResult(result)
     result.stopTest(self.test)
     result.core_time += self.test_duration
+    self.log_test_run_for_visualization()
+
+  def log_test_run_for_visualization(self):
+    if os.getenv('EMTEST_VISUALIZE') and (self.test_result != 'skipped' or self.test_duration > 0.2):
+      profiler_logs_path = os.path.join(tempfile.gettempdir(), 'emscripten_toolchain_profiler_logs')
+      os.makedirs(profiler_logs_path, exist_ok=True)
+      profiler_log_file = os.path.join(profiler_logs_path, 'toolchain_profiler.pid_0.json')
+      colors = {
+        'success': '#80ff80',
+        'warnings': '#ffb040',
+        'skipped': '#a0a0a0',
+        'expected failure': '#ff8080',
+        'unexpected success': '#ff8080',
+        'failed': '#ff8080',
+        'errored': '#ff8080',
+      }
+      # Write profiling entries for emprofile.py tool to visualize. This needs a unique identifier for each
+      # block, so generate one on the fly.
+      dummy_test_task_counter = os.path.getsize(profiler_log_file) if os.path.isfile(profiler_log_file) else 0
+      # Remove the redundant 'test_' prefix from each test, since character space is at a premium in the visualized graph.
+      test_name = self.test_short_name().removeprefix('test_')
+      with open(profiler_log_file, 'a') as prof:
+        prof.write(f',\n{{"pid":{dummy_test_task_counter},"op":"start","time":{self.start_time},"cmdLine":["{test_name}"],"color":"{colors[self.test_result]}"}}')
+        prof.write(f',\n{{"pid":{dummy_test_task_counter},"op":"exit","time":{self.start_time + self.test_duration},"returncode":0}}')
 
   def startTest(self, test):
     self.test_name = str(test)
@@ -198,37 +310,45 @@ class BufferedParallelTestResult:
     with self.lock:
       val = f'[{int(self.progress_counter.value * 100 / self.num_tests)}%] '
       self.progress_counter.value += 1
-    return val
+    return with_color(CYAN, val)
 
   def addSuccess(self, test):
-    print(f'{self.compute_progress()}{test} ... ok ({self.calculateElapsed():.2f}s)', file=sys.stderr)
+    msg = f'ok ({self.calculateElapsed():.2f}s)'
+    errlog(f'{self.compute_progress()}{test} ... {with_color(GREEN, msg)}')
     self.buffered_result = BufferedTestSuccess(test)
     self.test_result = 'success'
 
   def addExpectedFailure(self, test, err):
-    print(f'{self.compute_progress()}{test} ... expected failure ({self.calculateElapsed():.2f}s)', file=sys.stderr)
+    msg = f'expected failure ({self.calculateElapsed():.2f}s)'
+    errlog(f'{self.compute_progress()}{test} ... {with_color(RED, msg)}')
     self.buffered_result = BufferedTestExpectedFailure(test, err)
     self.test_result = 'expected failure'
 
   def addUnexpectedSuccess(self, test):
-    print(f'{self.compute_progress()}{test} ... unexpected success ({self.calculateElapsed():.2f}s)', file=sys.stderr)
+    msg = f'unexpected success ({self.calculateElapsed():.2f}s)'
+    errlog(f'{self.compute_progress()}{test} ... {with_color(RED, msg)}')
     self.buffered_result = BufferedTestUnexpectedSuccess(test)
     self.test_result = 'unexpected success'
 
   def addSkip(self, test, reason):
-    print(f"{self.compute_progress()}{test} ... skipped '{reason}'", file=sys.stderr)
+    msg = f"skipped '{reason}'"
+    errlog(f"{self.compute_progress()}{test} ... {with_color(CYAN, msg)}")
     self.buffered_result = BufferedTestSkip(test, reason)
     self.test_result = 'skipped'
 
   def addFailure(self, test, err):
-    print(f'{self.compute_progress()}{test} ... FAIL', file=sys.stderr)
+    msg = f'{test} ... FAIL'
+    errlog(f'{self.compute_progress()}{with_color(RED, msg)}')
     self.buffered_result = BufferedTestFailure(test, err)
     self.test_result = 'failed'
+    self.failures += [test]
 
   def addError(self, test, err):
-    print(f'{self.compute_progress()}{test} ... ERROR', file=sys.stderr)
+    msg = f'{test} ... ERROR'
+    errlog(f'{self.compute_progress()}{with_color(RED, msg)}')
     self.buffered_result = BufferedTestError(test, err)
     self.test_result = 'errored'
+    self.errors += [test]
 
 
 class BufferedTestBase:
@@ -336,4 +456,4 @@ class FakeCode:
 def num_cores():
   if NUM_CORES:
     return int(NUM_CORES)
-  return multiprocessing.cpu_count()
+  return utils.get_num_cores()
