@@ -111,7 +111,7 @@ def update_settings_glue(wasm_file, metadata, base_metadata):
     settings.WASM_EXPORTS = base_metadata.all_exports
   else:
     settings.WASM_EXPORTS = metadata.all_exports
-  settings.WASM_GLOBAL_EXPORTS = list(metadata.global_exports.keys())
+  settings.DATA_EXPORTS = list(metadata.data_exports.keys())
   settings.HAVE_EM_ASM = bool(settings.MAIN_MODULE or len(metadata.em_asm_consts) != 0)
 
   # start with the MVP features, and add any detected features.
@@ -269,13 +269,9 @@ def trim_asm_const_body(body):
   return body
 
 
-def create_other_export_declarations(tag_exports):
-  return '\n'.join(f'var {asmjs_mangle(name)};' for name in tag_exports)
-
-
-def create_global_exports(global_exports):
+def create_data_exports(data_exports):
   lines = []
-  for k, v in global_exports.items():
+  for k, v in data_exports.items():
     if shared.is_internal_global(k):
       continue
 
@@ -409,14 +405,14 @@ def emscript(in_wasm, out_wasm, outfile_js, js_syms, finalize=True, base_metadat
 
   if base_metadata:
     function_exports = base_metadata.function_exports
-    tag_exports = base_metadata.tag_exports
+    other_exports = base_metadata.other_exports
     # We want the real values from the final metadata but we only want to
     # include names from the base_metadata.  See phase_link() in link.py.
-    global_exports = {k: v for k, v in metadata.global_exports.items() if k in base_metadata.global_exports}
+    data_exports = {k: v for k, v in metadata.data_exports.items() if k in base_metadata.data_exports}
   else:
     function_exports = metadata.function_exports
-    tag_exports = metadata.tag_exports
-    global_exports = metadata.global_exports
+    other_exports = metadata.other_exports
+    data_exports = metadata.data_exports
 
   if settings.ASYNCIFY == 1:
     function_exports['asyncify_start_unwind'] = webassembly.FuncType([webassembly.Type.I32], [])
@@ -425,8 +421,10 @@ def emscript(in_wasm, out_wasm, outfile_js, js_syms, finalize=True, base_metadat
     function_exports['asyncify_stop_rewind'] = webassembly.FuncType([], [])
 
   parts = [pre]
-  parts += create_module(metadata, function_exports, global_exports, tag_exports, forwarded_json['librarySymbols'])
+  parts += create_module(metadata, function_exports, data_exports, other_exports,
+                         forwarded_json['librarySymbols'], forwarded_json['nativeAliases'])
   parts.append(post)
+  settings.ALIASES = list(forwarded_json['nativeAliases'].keys())
 
   full_js_module = ''.join(parts)
   full_js_module = apply_static_code_hooks(forwarded_json, full_js_module)
@@ -857,7 +855,7 @@ def create_sending(metadata, library_symbols):
       elems.append(f'{v} as {k}')
     elems = ',\n  '.join(elems)
     exports = '// Export JS functions to the wasm module with demangled names.\n'
-    exports += f"export {{\n  {elems}\n}};\n\n"
+    exports += f"export {{\n  {elems}\n}};"
     return exports
 
   prefix = ''
@@ -889,7 +887,7 @@ def create_reexports(metadata):
         wasm_exports.append(exp)
       elif demangled == 'main' and '__main_argc_argv' in settings.WASM_EXPORTS:
         wasm_exports.append('_main')
-  exports += f"export {{ {', '.join(wasm_exports)} }};\n\n"
+  exports += f"export {{ {', '.join(wasm_exports)} }};"
   return exports
 
 
@@ -910,12 +908,12 @@ def should_export(sym):
   return settings.EXPORT_ALL or (settings.EXPORT_KEEPALIVE and sym in settings.EXPORTED_FUNCTIONS)
 
 
-def create_receiving(function_exports, tag_exports, library_symbols):
+def create_receiving(function_exports, other_exports, library_symbols, aliases):
   generate_dyncall_assignment = 'dynCalls' in library_symbols
   receiving = ['\n// Imports from the Wasm binary.']
 
   if settings.WASM_ESM_INTEGRATION:
-    exported_symbols = tag_exports + list(function_exports.keys())
+    exported_symbols = other_exports + list(function_exports.keys())
     exports = []
     for sym in exported_symbols:
       mangled = asmjs_mangle(sym)
@@ -923,10 +921,9 @@ def create_receiving(function_exports, tag_exports, library_symbols):
         exports.append(f'{sym} as {mangled}')
       else:
         exports.append(sym)
-    if not settings.IMPORTED_MEMORY:
-      exports.append('memory as wasmMemory')
-    if not settings.RELOCATABLE:
-      exports.append('__indirect_function_table as wasmTable')
+    for alias, target in aliases.items():
+      exports.append(f'{target} as {alias}')
+
     receiving.append('import {')
     receiving.append('  ' + ',\n  '.join(exports))
     receiving.append(f"}} from './{settings.WASM_BINARY_FILE}';")
@@ -940,7 +937,7 @@ def create_receiving(function_exports, tag_exports, library_symbols):
           receiving.append(f"  dynCalls['{sig_str}'] = {sym};")
       receiving.append('}')
 
-    return '\n'.join(receiving) + '\n\n'
+    return '\n'.join(receiving)
 
   # When not declaring asm exports this section is empty and we instead programmatically export
   # symbols on the global object by calling exportWasmSymbols after initialization
@@ -953,78 +950,85 @@ def create_receiving(function_exports, tag_exports, library_symbols):
   # function assignWasmExports(wasmExport) {
   #   _main = wasmExports["_main"];
   exports = {name: sig for name, sig in function_exports.items() if name != building.WASM_CALL_CTORS}
+  for name in other_exports:
+    exports[name] = None
 
+  mangled = [asmjs_mangle(s) for s in exports] + list(aliases.keys())
   if settings.ASSERTIONS:
     # In debug builds we generate trapping functions in case
     # folks try to call/use a reference that was taken before the
     # wasm module is available.
-    for sym in exports:
-      mangled = asmjs_mangle(sym)
-      export_assignment = ''
-      if (settings.MODULARIZE or not settings.MINIMAL_RUNTIME) and should_export(mangled) and settings.MODULARIZE != 'instance':
-        export_assignment = f" = Module['{mangled}']"
-      receiving.append(f"var {mangled}{export_assignment} = makeInvalidEarlyAccess('{mangled}');")
+    for sym in mangled:
+      assignment = sym
+      if (settings.MODULARIZE or not settings.MINIMAL_RUNTIME) and should_export(sym) and settings.MODULARIZE != 'instance':
+        assignment += f" = Module['{sym}']"
+      receiving.append(f"var {assignment} = makeInvalidEarlyAccess('{sym}');")
   else:
     # Declare all exports in a single var statement
     sep = ',\n  '
-    mangled = [asmjs_mangle(s) for s in exports]
     receiving.append(f'var {sep.join(mangled)};\n')
 
   if settings.MODULARIZE == 'instance':
-    mangled = [asmjs_mangle(e) for e in exports]
     esm_exports = [e for e in mangled if should_export(e)]
     if esm_exports:
       esm_exports = ', '.join(esm_exports)
       receiving.append(f'export {{ {esm_exports} }};')
 
+  alias_inverse_map = {}
+  logger.debug(json.dumps(aliases))
+  for sym, alias in aliases.items():
+    assert alias in exports, f'expected alias target ({alias}) to be exported'
+    alias_inverse_map.setdefault(alias, []).append(sym)
+
+  do_module_exports = (settings.MODULARIZE or not settings.MINIMAL_RUNTIME) and settings.MODULARIZE != 'instance'
   receiving.append('\nfunction assignWasmExports(wasmExports) {')
   for sym, sig in exports.items():
+    is_function = sig is not None
     mangled = asmjs_mangle(sym)
-    if generate_dyncall_assignment and sym.startswith('dynCall_'):
+    assignment = mangled
+    if generate_dyncall_assignment and is_function and sym.startswith('dynCall_'):
       sig_str = sym.replace('dynCall_', '')
-      dynCallAssignment = f"dynCalls['{sig_str}'] = "
-    else:
-      dynCallAssignment = ''
-    export_assignment = ''
-    if (settings.MODULARIZE or not settings.MINIMAL_RUNTIME) and should_export(mangled) and settings.MODULARIZE != 'instance':
-      export_assignment = f"Module['{mangled}'] = "
-    if install_debug_wrapper(sym):
+      assignment += f" = dynCalls['{sig_str}']"
+    if do_module_exports and should_export(mangled):
+      assignment += f" = Module['{mangled}']"
+    if settings.ASSERTIONS:
+      receiving.append(f"  assert(wasmExports['{sym}'], 'missing Wasm export: {sym}');")
+    if sym in alias_inverse_map:
+      for target in alias_inverse_map[sym]:
+        assignment += f" = {target}"
+        if do_module_exports and target in settings.EXPORTED_RUNTIME_METHODS:
+          assignment += f" = Module['{target}']"
+    if is_function and install_debug_wrapper(sym):
       nargs = len(sig.params)
-      receiving.append(f"  {export_assignment}{dynCallAssignment}{mangled} = createExportWrapper('{sym}', {nargs});")
+      receiving.append(f"  {assignment} = createExportWrapper('{sym}', {nargs});")
     else:
-      receiving.append(f"  {export_assignment}{dynCallAssignment}{mangled} = wasmExports['{sym}'];")
+      receiving.append(f"  {assignment} = wasmExports['{sym}'];")
   receiving.append('}')
 
-  return '\n'.join(receiving) + '\n'
+  return '\n'.join(receiving)
 
 
-def create_module(metadata, function_exports, global_exports, tag_exports, library_symbols):
+def create_module(metadata, function_exports, data_exports, other_exports, library_symbols, aliases):
   module = []
+  module.append(create_receiving(function_exports, other_exports, library_symbols, aliases))
+  module.append(create_data_exports(data_exports))
 
-  receiving = create_receiving(function_exports, tag_exports, library_symbols)
-  receiving += create_global_exports(global_exports)
   sending = create_sending(metadata, library_symbols)
-
-  module.append(receiving)
-
   if settings.WASM_ESM_INTEGRATION:
     module.append(sending)
   else:
-    module.append(create_other_export_declarations(tag_exports))
-
     if settings.PTHREADS or settings.WASM_WORKERS or (settings.IMPORTED_MEMORY and settings.MODULARIZE == 'instance'):
       sending = textwrap.indent(sending, '  ').strip()
       module.append('''\
   var wasmImports;
   function assignWasmImports() {
     wasmImports = %s;
-  }
-  ''' % sending)
+  }''' % sending)
     else:
-      module.append('var wasmImports = %s;\n' % sending)
+      module.append('var wasmImports = %s;' % sending)
 
   if settings.SUPPORT_LONGJMP == 'emscripten' or not settings.DISABLE_EXCEPTION_CATCHING:
-    module.append(create_invoke_wrappers(metadata))
+    module += create_invoke_wrappers(metadata)
   else:
     assert not metadata.invoke_funcs, "invoke_ functions exported but exceptions and longjmp are both disabled"
 
@@ -1034,15 +1038,16 @@ def create_module(metadata, function_exports, global_exports, tag_exports, libra
   if settings.WASM_ESM_INTEGRATION:
     module.append(create_reexports(metadata))
 
-  return module
+  module = [chunk for chunk in module if chunk]
+  return '\n\n'.join(module) + '\n'
 
 
 def create_invoke_wrappers(metadata):
   """Asm.js-style exception handling: invoke wrapper generation."""
-  invoke_wrappers = ''
+  invoke_wrappers = []
   for invoke in metadata.invoke_funcs:
     sig = removeprefix(invoke, 'invoke_')
-    invoke_wrappers += '\n' + js_manipulation.make_invoke(sig) + '\n'
+    invoke_wrappers.append(js_manipulation.make_invoke(sig))
   return invoke_wrappers
 
 
